@@ -62,7 +62,7 @@
 
 G_DEFINE_TYPE (EBookBackendEws, e_book_backend_ews, E_TYPE_BOOK_BACKEND)
 
-static gboolean ebews_fetch_items (EBookBackendEws *ebews,  GSList *items, gboolean store_to_cache, GSList **vcards, GError **error);
+static gboolean ebews_fetch_items (EBookBackendEws *ebews,  GSList *items, gboolean store_to_cache, GSList **vcards, GCancellable *cancellable, GError **error);
 
 typedef struct {
 	GCond *cond;
@@ -98,6 +98,7 @@ struct _EBookBackendEwsPrivate {
 	SyncDelta *dlock;
 
 	ECredentials *credentials;
+	GCancellable *cancellable;
 };
 
 /* using this for backward compatibility with E_DATA_BOOK_MODE */
@@ -133,7 +134,7 @@ convert_error_to_edb_error (GError **perror)
 	if ((*perror)->domain == EWS_CONNECTION_ERROR) {
 		switch ((*perror)->code) {
 		case EWS_CONNECTION_ERROR_AUTHENTICATION_FAILED:
-			error = EDB_ERROR_EX (AUTHENTICATION_FAILED, (*perror)->message);
+			error = EDB_ERROR (AUTHENTICATION_FAILED);
 			break;
 		case EWS_CONNECTION_ERROR_CANCELLED:
 			break;
@@ -141,11 +142,11 @@ convert_error_to_edb_error (GError **perror)
 		case EWS_CONNECTION_ERROR_MANAGEDFOLDERNOTFOUND:
 		case EWS_CONNECTION_ERROR_PARENTFOLDERNOTFOUND:
 		case EWS_CONNECTION_ERROR_PUBLICFOLDERSERVERNOTFOUND:
-			error = EDB_ERROR_EX (NO_SUCH_BOOK, (*perror)->message);
+			error = EDB_ERROR (NO_SUCH_BOOK);
 			break;
 		case EWS_CONNECTION_ERROR_EVENTNOTFOUND:
 		case EWS_CONNECTION_ERROR_ITEMNOTFOUND:
-			error = EDB_ERROR_EX (CONTACT_NOT_FOUND, (*perror)->message);
+			error = EDB_ERROR (CONTACT_NOT_FOUND);
 			break;
 		}
 	}
@@ -897,8 +898,6 @@ ews_create_contact_cb (GObject *object,
 	/* get a list of ids from server (single item) */
 	e_ews_connection_create_items_finish (cnc, res, &items, &error);
 
-	g_return_if_fail (ebews->priv->ebsdb != NULL);
-
 	if (error == NULL) {
 		EEwsItem *item = (EEwsItem *) items->data;
 
@@ -1023,8 +1022,6 @@ ews_book_remove_contact_cb (GObject *object,
 
 	simple = G_SIMPLE_ASYNC_RESULT (res);
 
-	g_return_if_fail (priv->ebsdb != NULL);
-
 	if (!g_simple_async_result_propagate_error (simple, &error))
 		deleted = e_book_backend_sqlitedb_remove_contacts (priv->ebsdb, priv->folder_id, remove_contact->sl_ids, &error);
 
@@ -1123,8 +1120,6 @@ ews_modify_contact_cb (GObject *object,
 
 	e_ews_connection_update_items_finish (cnc, res, &items, &error);
 
-	g_return_if_fail (ebews->priv->ebsdb != NULL);
-
 	if (error == NULL) {
 		EEwsItem *item = (EEwsItem *) items->data;
 
@@ -1251,8 +1246,6 @@ e_book_backend_ews_modify_contacts (EBookBackend *backend,
 		return;
 	}
 
-	g_return_if_fail (priv->ebsdb != NULL);
-
 	contact = e_contact_new_from_vcard (vcards->data);
 
 	id = g_new0 (EwsId, 1);
@@ -1330,7 +1323,7 @@ e_book_backend_ews_get_contact_list (EBookBackend *backend,
 	priv = ebews->priv;
 
 	if (!priv->is_online) {
-		if (priv->ebsdb && e_book_backend_sqlitedb_get_is_populated (priv->ebsdb, priv->folder_id, NULL)) {
+		if (e_book_backend_sqlitedb_get_is_populated (priv->ebsdb, priv->folder_id, NULL)) {
 			list = e_book_backend_sqlitedb_search (priv->ebsdb, priv->folder_id, query, NULL, NULL, NULL, &error);
 			l = list;
 			while (l) {
@@ -1357,7 +1350,7 @@ e_book_backend_ews_get_contact_list (EBookBackend *backend,
 		return;
 	}
 
-	if (priv->ebsdb && e_book_backend_sqlitedb_get_is_populated (priv->ebsdb, priv->folder_id, NULL)) {
+	if (e_book_backend_sqlitedb_get_is_populated (priv->ebsdb, priv->folder_id, NULL)) {
 		list = e_book_backend_sqlitedb_search (priv->ebsdb, priv->folder_id, query, NULL, NULL, NULL, &error);
 		l = list;
 		while (l) {
@@ -1390,10 +1383,10 @@ e_book_backend_ews_get_contact_list (EBookBackend *backend,
 							EWS_FOLDER_TYPE_CONTACTS,
 							&includes_last_item,
 							&items, (EwsConvertQueryCallback) (e_ews_query_to_restriction),
-							NULL, &error);
+							cancellable, &error);
 
 		/*we have got Id for items lets fetch them using getitem operation*/
-		ebews_fetch_items (ebews, items, FALSE, &vcard_list, &error);
+		ebews_fetch_items (ebews, items, FALSE, &vcard_list, cancellable, &error);
 		convert_error_to_edb_error (&error);
 		e_data_book_respond_get_contact_list (book, opid, error, vcard_list);
 
@@ -1620,9 +1613,6 @@ ews_gal_needs_update (EBookBackendEws *cbews,
 	gboolean ret = FALSE;
 	gchar *tmp;
 
-	if (!priv->ebsdb)
-		return FALSE;
-
 	tmp = e_book_backend_sqlitedb_get_key_value (priv->ebsdb, priv->folder_id, "seq", error);
 	if (error)
 		goto exit;
@@ -1637,15 +1627,6 @@ exit:
 	return ret;
 }
 
-static gboolean
-ews_book_backend_get_use_ntlm (EBookBackendEws *cbews)
-{
-	ESource *source;
-
-	source = e_backend_get_source (E_BACKEND (cbews));
-	return g_strcmp0 ("PLAIN", e_source_get_property (source, "ews-auth-type")) != 0;
-}
-
 static gchar *
 ews_download_full_gal (EBookBackendEws *cbews,
                        EwsOALDetails *full,
@@ -1664,7 +1645,7 @@ ews_download_full_gal (EBookBackendEws *cbews,
 	cache_dir = e_book_backend_get_cache_dir (E_BOOK_BACKEND (cbews));
 	comp_cache_file = g_build_filename (cache_dir, full->filename, NULL);
 
-	oab_cnc = e_ews_connection_new (full_url, priv->username, priv->password, ews_book_backend_get_use_ntlm (cbews), NULL, NULL, NULL);
+	oab_cnc = e_ews_connection_new (full_url, priv->username, priv->password, NULL, NULL, NULL);
 	if (!e_ews_connection_download_oal_file (oab_cnc, comp_cache_file, NULL, NULL, cancellable, error))
 		goto exit;
 
@@ -1697,16 +1678,12 @@ ews_remove_old_gal_file (EBookBackendEws *cbews,
 	EBookBackendEwsPrivate *priv = cbews->priv;
 	gchar *filename;
 
-	if (!priv->ebsdb)
-		return FALSE;
-
 	filename = e_book_backend_sqlitedb_get_key_value (priv->ebsdb, priv->folder_id, "oab-filename", error);
 	if (*error)
 		return FALSE;
 
 	if (filename)
 		g_unlink (filename);
-	g_free (filename);
 
 	return TRUE;
 }
@@ -1727,8 +1704,6 @@ ews_gal_store_contact (EContact *contact,
 	struct _db_data *data = (struct _db_data *) user_data;
 	EBookBackendEwsPrivate *priv = data->cbews->priv;
 
-	g_return_if_fail (priv->ebsdb != NULL);
-
 	data->contact_collector = g_slist_prepend (data->contact_collector, g_object_ref (contact));
 	data->collected_length += 1;
 
@@ -1769,8 +1744,6 @@ ews_replace_gal_in_db (EBookBackendEws *cbews,
 	gboolean ret = TRUE;
 	struct _db_data data;
 
-	g_return_val_if_fail (priv->ebsdb != NULL, FALSE);
-
 	/* remove the old address-book and create a new one in db */
 	if (e_book_backend_sqlitedb_get_is_populated (priv->ebsdb, priv->folder_id, NULL)) {
 		ret = e_book_backend_sqlitedb_delete_addressbook (priv->ebsdb, priv->folder_id, error);
@@ -1811,32 +1784,26 @@ ebews_start_gal_sync (gpointer data)
 	GSList *full_l = NULL;
 	gboolean ret = TRUE;
 	gchar *uncompressed_filename = NULL;
-	GCancellable *cancellable;
 
 	cbews = (EBookBackendEws *) data;
 	priv = cbews->priv;
 
-	cancellable = g_cancellable_new ();
-	oab_cnc = e_ews_connection_new (priv->oab_url, priv->username, priv->password, ews_book_backend_get_use_ntlm (cbews), NULL, NULL, NULL);
+	oab_cnc = e_ews_connection_new (priv->oab_url, priv->username, priv->password, NULL, NULL, NULL);
 
 	d(printf ("Ewsgal: Fetching oal full details file \n");)
 
-	if (!e_ews_connection_get_oal_detail (oab_cnc, priv->folder_id, "Full", &full_l, cancellable, &error)) {
+	if (!e_ews_connection_get_oal_detail (oab_cnc, priv->folder_id, "Full", &full_l, priv->cancellable, &error)) {
 		ret = FALSE;
 		goto exit;
 	}
 
-	g_warn_if_fail (priv->ebsdb != NULL);
-	if (!priv->ebsdb)
-		goto exit;
-
 	full = (EwsOALDetails *) full_l->data;
 	/* TODO fetch differential updates if available instead of downloading the whole GAL */
 	if (!e_book_backend_sqlitedb_get_is_populated (priv->ebsdb, priv->folder_id, NULL) || ews_gal_needs_update (cbews, full, &error)) {
 		gchar *seq;
 
 		d(printf ("Ewsgal: Downloading full gal \n");)
-		uncompressed_filename = ews_download_full_gal (cbews, full, cancellable, &error);
+		uncompressed_filename = ews_download_full_gal (cbews, full, priv->cancellable, &error);
 		if (error) {
 			ret = FALSE;
 			goto exit;
@@ -1850,7 +1817,7 @@ ebews_start_gal_sync (gpointer data)
 		}
 
 		d(printf ("Ewsgal: Replacing old gal with new gal contents in db \n");)
-		ret = ews_replace_gal_in_db (cbews, uncompressed_filename, cancellable, &error);
+		ret = ews_replace_gal_in_db (cbews, uncompressed_filename, priv->cancellable, &error);
 		if (!ret)
 			goto exit;
 
@@ -1907,8 +1874,6 @@ ebews_sync_deleted_items (EBookBackendEws *ebews,
 
 	priv = ebews->priv;
 
-	g_return_if_fail (priv->ebsdb != NULL);
-
 	for (l = deleted_ids; l != NULL; l = g_slist_next (l)) {
 		gchar *id = (gchar *) l->data;
 		gboolean partial_content;
@@ -1933,8 +1898,6 @@ ebews_store_contact_items (EBookBackendEws *ebews,
 
 	priv = ebews->priv;
 
-	g_return_if_fail (priv->ebsdb != NULL);
-
 	for (l = new_items; l != NULL; l = g_slist_next (l)) {
 		EContact *contact;
 		gint i, element_type;
@@ -2012,6 +1975,23 @@ ebews_get_vcards_list (GSList *new_items,
 	g_slist_free (new_items);
 }
 
+static void
+ews_mb_free (EwsMailbox *mb)
+{
+	if (mb) {
+		g_free (mb->name);
+		g_free (mb->email);
+
+		if (mb->item_id) {
+			g_free (mb->item_id->id);
+			g_free (mb->item_id->change_key);
+			g_free (mb->item_id);
+		}
+
+		g_free (mb);
+	}
+}
+
 static void
 ebews_store_distribution_list_items (EBookBackendEws *ebews,
                                      const EwsId *id,
@@ -2022,8 +2002,6 @@ ebews_store_distribution_list_items (EBookBackendEws *ebews,
 	GSList *l;
 	EContact *contact;
 
-	g_return_if_fail (ebews->priv->ebsdb != NULL);
-
 	contact = e_contact_new ();
 	e_contact_set (contact, E_CONTACT_UID, id->id);
 	e_contact_set (contact, E_CONTACT_REV, id->change_key);
@@ -2052,7 +2030,7 @@ ebews_store_distribution_list_items (EBookBackendEws *ebews,
 			e_vcard_attribute_add_value (attr, mb->email);
 
 		e_vcard_add_attribute (E_VCARD (contact), attr);
-		e_ews_mailbox_free (mb);
+		ews_mb_free (mb);
 	}
 
 	g_slist_free (members);
@@ -2100,7 +2078,7 @@ ebews_vcards_append_dl (const EwsId *id,
 			e_vcard_attribute_add_value (attr, mb->email);
 
 		e_vcard_add_attribute (E_VCARD (contact), attr);
-		e_ews_mailbox_free (mb);
+		ews_mb_free (mb);
 	}
 	vcard_string = e_vcard_to_string (E_VCARD (contact), EVC_FORMAT_VCARD_30);
 	*vcards = g_slist_append (*vcards, g_strdup(vcard_string));
@@ -2114,6 +2092,7 @@ ebews_fetch_items (EBookBackendEws *ebews,
                    GSList *items,
                    gboolean store_to_cache,
                    GSList **vcards,
+		   GCancellable *cancellable,
                    GError **error)
 {
 	EBookBackendEwsPrivate *priv;
@@ -2147,7 +2126,7 @@ ebews_fetch_items (EBookBackendEws *ebews,
 			(cnc, EWS_PRIORITY_MEDIUM,
 			 contact_item_ids, "Default", CONTACT_ITEM_PROPS,
 			 FALSE, NULL, &new_items, NULL, NULL,
-			 NULL, error);
+			 cancellable, error);
 	if (*error)
 		goto cleanup;
 
@@ -2165,7 +2144,7 @@ ebews_fetch_items (EBookBackendEws *ebews,
 			(cnc, EWS_PRIORITY_MEDIUM,
 			 dl_ids, "Default", NULL,
 			 FALSE, NULL, &new_items, NULL, NULL,
-			 NULL, error);
+			 cancellable, error);
 	if (*error)
 		goto cleanup;
 
@@ -2189,7 +2168,7 @@ ebews_fetch_items (EBookBackendEws *ebews,
 			goto cleanup;
 
 		d_name = e_ews_item_get_subject (item);
-		e_ews_connection_expand_dl (cnc, EWS_PRIORITY_MEDIUM, mb, &members, &includes_last, NULL, error);
+		e_ews_connection_expand_dl (cnc, EWS_PRIORITY_MEDIUM, mb, &members, &includes_last, priv->cancellable, error);
 		if (*error)
 			goto cleanup;
 
@@ -2235,8 +2214,6 @@ ebews_start_sync (gpointer data)
 	ebews = (EBookBackendEws *) data;
 	priv = ebews->priv;
 
-	g_return_val_if_fail (priv->ebsdb != NULL, FALSE);
-
 	status_message = g_strdup (_("Syncing contacts..."));
 	e_book_backend_foreach_view (E_BOOK_BACKEND (ebews), book_view_notify_status, status_message);
 	g_free (status_message);
@@ -2252,7 +2229,7 @@ ebews_start_sync (gpointer data)
 							 "IdOnly", NULL,
 							 EWS_MAX_FETCH_COUNT, &includes_last_item,
 							 &items_created, &items_updated,
-							 &items_deleted, NULL, &error);
+							 &items_deleted, priv->cancellable, &error);
 
 		if (error)
 			break;
@@ -2261,7 +2238,7 @@ ebews_start_sync (gpointer data)
 			ebews_sync_deleted_items (ebews, items_deleted, &error);
 
 		if (items_created)
-			ebews_fetch_items (ebews, items_created, TRUE, NULL, &error);
+			ebews_fetch_items (ebews, items_created, TRUE, NULL, priv->cancellable, &error);
 
 		if (error) {
 			if (items_updated) {
@@ -2273,7 +2250,7 @@ ebews_start_sync (gpointer data)
 		}
 
 		if (items_updated)
-			ebews_fetch_items (ebews, items_updated, TRUE, NULL, &error);
+			ebews_fetch_items (ebews, items_updated, TRUE, NULL, priv->cancellable, &error);
 
 		if (error)
 			break;
@@ -2286,9 +2263,6 @@ ebews_start_sync (gpointer data)
 
 	g_free (sync_state);
 
-	/* hide progress message when done */
-	e_book_backend_foreach_view (E_BOOK_BACKEND (ebews), book_view_notify_status, NULL);
-
 	if (error) {
 		g_warning ("Error Syncing Contacts: Folder %s Error: %s", priv->folder_id, error->message);
 		g_clear_error (&error);
@@ -2394,8 +2368,6 @@ fetch_from_offline (EBookBackendEws *ews,
 		return;
 	}
 
-	g_return_if_fail (priv->ebsdb != NULL);
-
 	contacts = e_book_backend_sqlitedb_search (priv->ebsdb, priv->folder_id, query, NULL, NULL, NULL, &error);
 	for (l = contacts; l != NULL; l = g_slist_next (l)) {
 		EbSdbSearchData *s_data = (EbSdbSearchData *) l->data;
@@ -2538,7 +2510,9 @@ e_book_backend_ews_start_book_view (EBookBackend *backend,
 
 		e_data_book_view_notify_update (book_view, contact);
 
-		e_ews_mailbox_free (mb);
+		g_free (mb->email);
+		g_free (mb->name);
+		g_free (mb);
 		e_ews_free_resolve_contact (rc);
 		g_object_unref (contact);
 	}
@@ -2602,8 +2576,6 @@ e_book_backend_ews_load_source (EBookBackend *backend,
 			priv->marked_for_offline = TRUE;
 	} else {
 		priv->folder_id = e_source_get_duped_property (source, "oal_id");
-		priv->marked_for_offline = FALSE;
-		priv->is_writable = FALSE;
 
 		/* If folder_id is present it means the GAL is marked for offline usage, we do not check for offline_sync property */
 		if (priv->folder_id) {
@@ -2620,6 +2592,7 @@ e_book_backend_ews_load_source (EBookBackend *backend,
 				return;
 			}
 			priv->marked_for_offline = TRUE;
+			priv->is_writable = FALSE;
 		}
 	}
 
@@ -2667,19 +2640,12 @@ e_book_backend_ews_authenticate_user (EBookBackend *backend,
 		return;
 	}
 
-	if (!credentials || !e_credentials_peek (credentials, E_CREDENTIALS_KEY_USERNAME)) {
-		e_book_backend_notify_opened (backend, EDB_ERROR (AUTHENTICATION_REQUIRED));
-		e_book_backend_notify_readonly (backend, TRUE);
-		return;
-	}
-
 	esource = e_backend_get_source (E_BACKEND (backend));
 	host_url = e_source_get_property (esource, "hosturl");
 	read_only = e_source_get_property (esource, "read_only");
 
 	cnc = e_ews_connection_new (host_url, e_credentials_peek (credentials, E_CREDENTIALS_KEY_USERNAME),
 					  e_credentials_peek (credentials, E_CREDENTIALS_KEY_PASSWORD),
-					  ews_book_backend_get_use_ntlm (ebews),
 					  NULL, NULL, &error);
 
 	if ((read_only && !strcmp (read_only, "true")) || priv->is_gal) {
@@ -2693,7 +2659,7 @@ e_book_backend_ews_authenticate_user (EBookBackend *backend,
 		fid->id = g_strdup ("contacts");
 		fid->is_distinguished_id = TRUE;
 		ids = g_slist_append (ids, fid);
-		e_ews_connection_get_folder (cnc, EWS_PRIORITY_MEDIUM, "Default", NULL, ids, &folders, NULL, &error);
+		e_ews_connection_get_folder (cnc, EWS_PRIORITY_MEDIUM, "Default", NULL, ids, &folders, cancellable, &error);
 
 		e_ews_folder_free_fid (fid);
 		g_slist_free (ids);
@@ -2734,6 +2700,12 @@ e_book_backend_ews_notify_online_cb (EBookBackend *backend,
 	ebews->priv->is_online = is_online;
 
 	if (e_book_backend_is_opened (backend)) {
+		if (ebews->priv->cancellable) {
+			g_cancellable_cancel (ebews->priv->cancellable);
+			g_object_unref (ebews->priv->cancellable);
+			ebews->priv->cancellable = NULL;
+		}
+
 		if (!is_online) {
 			e_book_backend_notify_readonly (backend, TRUE);
 			e_book_backend_notify_online (backend, FALSE);
@@ -2742,6 +2714,8 @@ e_book_backend_ews_notify_online_cb (EBookBackend *backend,
 				ebews->priv->cnc = NULL;
 			}
 		} else {
+			ebews->priv->cancellable = g_cancellable_new ();
+
 			e_book_backend_notify_readonly (backend, !ebews->priv->is_writable);
 			e_book_backend_notify_online (backend, TRUE);
 			e_book_backend_notify_auth_required (backend, TRUE, NULL);
@@ -2849,6 +2823,12 @@ e_book_backend_ews_dispose (GObject *object)
 	bews = E_BOOK_BACKEND_EWS (object);
 	priv = bews->priv;
 
+	if (priv->cancellable) {
+		g_cancellable_cancel (priv->cancellable);
+		g_object_unref (priv->cancellable);
+		priv->cancellable = NULL;
+	}
+
 	if (priv->cnc) {
 		g_object_unref (priv->cnc);
 		priv->cnc = NULL;
@@ -2955,6 +2935,7 @@ e_book_backend_ews_init (EBookBackendEws *backend)
 
 	bews->priv = priv;
 	g_static_rec_mutex_init (&priv->rec_mutex);
+	priv->cancellable = g_cancellable_new ();
 
 	g_signal_connect (
 		bews, "notify::online",
