@@ -154,42 +154,15 @@ ews_set_alarm (ESoapMessage *msg,
 void
 ewscal_set_time (ESoapMessage *msg,
                  const gchar *name,
-                 icaltimetype *t,
-		 gboolean with_timezone)
+                 icaltimetype *t)
 {
 	gchar *str;
-	gchar *tz_ident = NULL;
 
-	if (with_timezone) {
-		if (t->is_utc || !t->zone || t->zone == icaltimezone_get_utc_timezone ()) {
-			tz_ident = g_strdup ("Z");
-		} else {
-			gint offset, is_daylight, hrs, mins;
-
-			offset = icaltimezone_get_utc_offset (
-				icaltimezone_get_utc_timezone (), t, &is_daylight);
-
-			offset = offset * (-1);
-			hrs = offset / 60;
-			mins = offset % 60;
-
-			if (hrs < 0)
-				hrs *= -1;
-			if (mins < 0)
-				mins *= -1;
-
-			tz_ident = g_strdup_printf ("%s%02d:%02d", offset > 0 ? "+" : "-", hrs, mins);
-		}
-	}
-
-	str = g_strdup_printf ("%04d-%02d-%02dT%02d:%02d:%02d%s",
-			       t->year, t->month, t->day,
-			       t->hour, t->minute, t->second,
-			       tz_ident ? tz_ident : "");
+	str = g_strdup_printf("%04d-%02d-%02dT%02d:%02d:%02d",
+			      t->year, t->month, t->day,
+			      t->hour, t->minute, t->second);
 
 	e_ews_message_write_string_parameter (msg, name, NULL, str);
-
-	g_free (tz_ident);
 	g_free (str);
 }
 
@@ -399,6 +372,27 @@ ewscal_add_availability_rrule (ESoapMessage *msg,
 	e_ews_message_write_string_parameter(msg, "DayOfWeek", NULL, number_to_weekday(icalrecurrencetype_day_day_of_week(recur.by_day[0])));
 }
 
+static void
+ewscal_add_availability_default_timechange (ESoapMessage *msg)
+{
+
+	e_soap_message_start_element(msg, "StandardTime", NULL, NULL);
+	e_ews_message_write_string_parameter(msg, "Bias", NULL, "0");
+	e_ews_message_write_string_parameter(msg, "Time", NULL, "00:00:00");
+	e_ews_message_write_string_parameter(msg, "DayOrder", NULL, "0");
+	e_ews_message_write_string_parameter(msg, "Month", NULL, "0");
+	e_ews_message_write_string_parameter(msg, "DayOfWeek", NULL, "Sunday");
+	e_soap_message_end_element (msg);
+
+	e_soap_message_start_element(msg, "DaylightTime", NULL, NULL);
+	e_ews_message_write_string_parameter(msg, "Bias", NULL, "0");
+	e_ews_message_write_string_parameter(msg, "Time", NULL, "00:00:00");
+	e_ews_message_write_string_parameter(msg, "DayOrder", NULL, "0");
+	e_ews_message_write_string_parameter(msg, "Month", NULL, "0");
+	e_ews_message_write_string_parameter(msg, "DayOfWeek", NULL, "Sunday");
+	e_soap_message_end_element (msg);
+}
+
 static void
 ewscal_add_availability_timechange (ESoapMessage *msg,
                                     icalcomponent *comp,
@@ -448,16 +442,16 @@ ewscal_set_availability_timezone (ESoapMessage *msg,
 	xstd = icalcomponent_get_first_component (comp, ICAL_XSTANDARD_COMPONENT);
 	xdaylight = icalcomponent_get_first_component (comp, ICAL_XDAYLIGHT_COMPONENT);
 
-	/* Should never happen. Exchange will bail out */
-	if (!xstd || !xdaylight)
-		return;
-
+	/*TimeZone is the root element of GetUserAvailabilityRequest*/
 	e_soap_message_start_element(msg, "TimeZone", NULL, NULL);
 
 	/* Fetch the timezone offsets for the standard (or only) zone.
 	 * Negate it, because Exchange does it backwards */
-	prop = icalcomponent_get_first_property (xstd, ICAL_TZOFFSETTO_PROPERTY);
-	std_utcoffs = -icalproperty_get_tzoffsetto (prop) / 60;
+	if (xstd) {
+		prop = icalcomponent_get_first_property(xstd, ICAL_TZOFFSETTO_PROPERTY);
+		std_utcoffs = -icalproperty_get_tzoffsetto(prop)/60;
+	} else
+		std_utcoffs = 0;
 
 	/* This is the overall BaseOffset tag, which the Standard and Daylight
 	 * zones are offset from. It's redundant, but Exchange always sets it
@@ -477,7 +471,10 @@ ewscal_set_availability_timezone (ESoapMessage *msg,
 		e_soap_message_start_element(msg, "DaylightTime", NULL, NULL);
 		ewscal_add_availability_timechange (msg, xdaylight, std_utcoffs);
 		e_soap_message_end_element(msg); /* "DaylightTime" */
-	}
+	} else 
+		/* Set default values*/
+		ewscal_add_availability_default_timechange (msg);
+
 	e_soap_message_end_element(msg); /* "TimeZone" */
 }
 
