@@ -90,8 +90,8 @@ struct _CamelEwsFolderPrivate {
 
 extern gint camel_application_is_exiting;
 
-static gboolean ews_delete_messages (CamelFolder *folder, GSList *deleted_items, gboolean expunge, GCancellable *cancellable, GError **error);
-static gboolean ews_refresh_info_sync (CamelFolder *folder, GCancellable *cancellable, GError **error);
+static gboolean
+ews_delete_messages (CamelFolder *folder, GSList *deleted_items, gboolean expunge, GCancellable *cancellable, GError **error);
 
 #define d(x)
 
@@ -390,7 +390,6 @@ camel_ews_folder_get_message (CamelFolder *folder,
 	const gchar *temp;
 	gboolean res;
 	gchar *mime_fname_new = NULL;
-	GError *local_error = NULL;
 
 	ews_store = (CamelEwsStore *) camel_folder_get_parent_store (folder);
 	ews_folder = (CamelEwsFolder *) folder;
@@ -444,14 +443,11 @@ camel_ews_folder_get_message (CamelFolder *folder,
 					  &items,
 					  (ESoapProgressFn) camel_operation_progress,
 					  (gpointer) cancellable,
-					  cancellable, &local_error);
+					  cancellable, error);
 	g_free (mime_dir);
 
-	if (!res || !items) {
-		camel_ews_store_maybe_disconnect (ews_store, local_error);
-		g_propagate_error (error, local_error);
+	if (!res)
 		goto exit;
-	}
 
 	/* The mime_content actually contains the *filename*, due to the
 	 * streaming hack in ESoapMessage */
@@ -475,16 +471,12 @@ camel_ews_folder_get_message (CamelFolder *folder,
 						  &items_req,
 						  (ESoapProgressFn) camel_operation_progress,
 						  (gpointer) cancellable,
-						  cancellable, &local_error);
+						  cancellable, error);
 		if (!res || (items_req && e_ews_item_get_item_type (items_req->data) == E_EWS_ITEM_TYPE_ERROR)) {
 			if (items_req) {
 				g_object_unref (items_req->data);
 				g_slist_free (items_req);
 			}
-			if (local_error) {
-				camel_ews_store_maybe_disconnect (ews_store, local_error);
-				g_propagate_error (error, local_error);
-			}
 			goto exit;
 		}
 		calendar_item_accept_id = e_ews_item_get_calendar_item_accept_id (items_req->data);
@@ -549,7 +541,6 @@ exit:
 
 	if (mime_fname_new)
 		g_free (mime_fname_new);
-	g_object_unref (cnc);
 
 	return message;
 }
@@ -573,6 +564,7 @@ ews_folder_get_message_sync (CamelFolder *folder,
 static GPtrArray *
 ews_folder_search_by_expression (CamelFolder *folder,
                                  const gchar *expression,
+				 GCancellable *cancellable,
                                  GError **error)
 {
 	CamelEwsFolder *ews_folder;
@@ -585,7 +577,7 @@ ews_folder_search_by_expression (CamelFolder *folder,
 	g_mutex_lock (priv->search_lock);
 
 	camel_folder_search_set_folder (ews_folder->search, folder);
-	matches = camel_folder_search_search (ews_folder->search, expression, NULL, error);
+	matches = camel_folder_search_search (ews_folder->search, expression, NULL, cancellable, error);
 
 	g_mutex_unlock (priv->search_lock);
 
@@ -595,6 +587,7 @@ ews_folder_search_by_expression (CamelFolder *folder,
 static guint32
 ews_folder_count_by_expression (CamelFolder *folder,
                                 const gchar *expression,
+				GCancellable *cancellable,
                                 GError **error)
 {
 	CamelEwsFolder *ews_folder;
@@ -607,7 +600,7 @@ ews_folder_count_by_expression (CamelFolder *folder,
 	g_mutex_lock (priv->search_lock);
 
 	camel_folder_search_set_folder (ews_folder->search, folder);
-	matches = camel_folder_search_count (ews_folder->search, expression, error);
+	matches = camel_folder_search_count (ews_folder->search, expression, cancellable, error);
 
 	g_mutex_unlock (priv->search_lock);
 
@@ -618,6 +611,7 @@ static GPtrArray *
 ews_folder_search_by_uids (CamelFolder *folder,
                            const gchar *expression,
                            GPtrArray *uids,
+			   GCancellable *cancellable,
                            GError **error)
 {
 	CamelEwsFolder *ews_folder;
@@ -633,7 +627,7 @@ ews_folder_search_by_uids (CamelFolder *folder,
 	g_mutex_lock (priv->search_lock);
 
 	camel_folder_search_set_folder (ews_folder->search, folder);
-	matches = camel_folder_search_search (ews_folder->search, expression, uids, error);
+	matches = camel_folder_search_search (ews_folder->search, expression, uids, cancellable, error);
 
 	g_mutex_unlock (priv->search_lock);
 
@@ -783,40 +777,23 @@ ews_sync_mi_flags (CamelFolder *folder,
 {
 	CamelEwsStore *ews_store;
 	EEwsConnection *cnc;
-	GError *local_error = NULL;
-	gboolean res;
 
 	ews_store = (CamelEwsStore *) camel_folder_get_parent_store (folder);
-
-	if (!camel_ews_store_connected (ews_store, error)) {
-		return FALSE;
-	}
-
 	cnc = camel_ews_store_get_connection (ews_store);
 
-	res = e_ews_connection_update_items (cnc, EWS_PRIORITY_LOW,
+	return e_ews_connection_update_items (cnc, EWS_PRIORITY_LOW,
 					      "AlwaysOverwrite", "SaveOnly",
 					      NULL, NULL,
 					      msg_update_flags, mi_list, NULL,
-					      cancellable, &local_error);
-
-	if (local_error) {
-		camel_ews_store_maybe_disconnect (ews_store, local_error);
-		g_propagate_error (error, local_error);
-	}
-
-	g_object_unref (cnc);
-
-	return res;
+					      cancellable, error);
 }
 
 static gboolean
-ews_folder_is_of_type (CamelFolder *folder,
-		       guint32 folder_type)
+ews_folder_is_trash (CamelFolder *folder)
 {
 	CamelStore *parent_store;
 	CamelEwsStore *ews_store;
-	gboolean is_of_type;
+	gboolean is_deleted_items;
 	gchar *folder_id;
 
 	g_return_val_if_fail (folder != NULL, FALSE);
@@ -826,104 +803,12 @@ ews_folder_is_of_type (CamelFolder *folder,
 
 	g_return_val_if_fail (ews_store != NULL, FALSE);
 
-	folder_type = folder_type & CAMEL_FOLDER_TYPE_MASK;
 	folder_id = camel_ews_store_summary_get_folder_id_from_name (ews_store->summary, camel_folder_get_full_name (folder));
-	is_of_type = folder_id &&
-		(camel_ews_store_summary_get_folder_flags (ews_store->summary, folder_id, NULL) & CAMEL_FOLDER_TYPE_MASK) == folder_type;
+	is_deleted_items = folder_id &&
+		(camel_ews_store_summary_get_folder_flags (ews_store->summary, folder_id, NULL) & CAMEL_FOLDER_TYPE_MASK) == CAMEL_FOLDER_TYPE_TRASH;
 	g_free (folder_id);
 
-	return is_of_type;
-}
-
-static gboolean
-ews_move_to_junk_folder (CamelFolder *folder,
-			 GSList *junk_uids,
-			 GCancellable *cancellable,
-			 GError **error)
-{
-	CamelEwsFolder *ews_folder;
-	CamelStore *parent_store;
-	CamelEwsStore *ews_store;
-	EEwsConnection *cnc;
-	gboolean status = TRUE;
-
-	if (ews_folder_is_of_type (folder, CAMEL_FOLDER_TYPE_JUNK)) {
-		/* cannot move to itself, but treat it as success */
-		g_slist_free_full (junk_uids, (GDestroyNotify) camel_pstring_free);
-		return TRUE;
-	}
-
-	ews_store = CAMEL_EWS_STORE (camel_folder_get_parent_store (folder));
-
-	parent_store = camel_folder_get_parent_store (folder);
-	ews_folder = CAMEL_EWS_FOLDER (folder);
-	ews_store = CAMEL_EWS_STORE (parent_store);
-
-	if (!camel_ews_store_connected (ews_store, error))
-		return FALSE;
-
-	cnc = camel_ews_store_get_connection (ews_store);
-
-	if (junk_uids) {
-		GSList *moved_items = NULL;
-		GError *local_error = NULL;
-		gchar *folder_id;
-
-		folder_id = camel_ews_store_summary_get_folder_id_from_folder_type (
-			ews_store->summary, CAMEL_FOLDER_TYPE_JUNK);
-
-		camel_service_lock (CAMEL_SERVICE (ews_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
-		status = e_ews_connection_move_items (cnc, EWS_PRIORITY_MEDIUM, folder_id, FALSE,
-			junk_uids, &moved_items, cancellable, &local_error);
-		camel_service_unlock (CAMEL_SERVICE (ews_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
-
-		if (!status && local_error && local_error->code == EWS_CONNECTION_ERROR_ITEMNOTFOUND) {
-			/* If move failed due to the item not found, ignore the error,
-			 * trigger folder info refresh and then go on to clear the
-			 * cache of the remove items anyway. */
-			g_clear_error (&local_error);
-			status = ews_refresh_info_sync (folder, cancellable, &local_error);
-		}
-
-		g_slist_free_full (moved_items, g_object_unref);
-		g_free (folder_id);
-
-		if (status) {
-			CamelFolderChangeInfo *changes;
-			GSList *iter;
-
-			changes = camel_folder_change_info_new ();
-
-			for (iter = junk_uids; iter; iter = g_slist_next (iter)) {
-				const gchar *uid = iter->data;
-
-				camel_folder_summary_lock (folder->summary, CAMEL_FOLDER_SUMMARY_SUMMARY_LOCK);
-
-				camel_folder_change_info_remove_uid (changes, uid);
-				camel_folder_summary_remove_uid (folder->summary, uid);
-				ews_data_cache_remove (ews_folder->cache, "cur", uid, NULL);
-
-				camel_folder_summary_unlock (folder->summary, CAMEL_FOLDER_SUMMARY_SUMMARY_LOCK);
-			}
-
-			if (camel_folder_change_info_changed (changes)) {
-				camel_folder_summary_touch (folder->summary);
-				camel_folder_changed (folder, changes);
-			}
-			camel_folder_change_info_free (changes);
-		}
-
-		if (local_error) {
-			camel_ews_store_maybe_disconnect (ews_store, local_error);
-			g_propagate_error (error, local_error);
-		}
-
-		g_slist_free_full (junk_uids, (GDestroyNotify) camel_pstring_free);
-	}
-
-	g_object_unref (cnc);
-
-	return status;
+	return is_deleted_items;
 }
 
 static gboolean
@@ -934,7 +819,7 @@ ews_synchronize_sync (CamelFolder *folder,
 {
 	CamelEwsStore *ews_store;
 	GPtrArray *uids;
-	GSList *mi_list = NULL, *deleted_uids = NULL, *junk_uids = NULL;
+	GSList *mi_list = NULL, *deleted_uids = NULL;
 	gint mi_list_len = 0;
 	gboolean success = TRUE;
 	gint i;
@@ -944,50 +829,35 @@ ews_synchronize_sync (CamelFolder *folder,
 	if (!camel_ews_store_connected (ews_store, error))
 		return FALSE;
 
-	if (camel_folder_summary_get_deleted_count (folder->summary) > 0 ||
-	    camel_folder_summary_get_junk_count (folder->summary) > 0) {
-		camel_folder_summary_prepare_fetch_all (folder->summary, NULL);
-		uids = camel_folder_summary_get_array (folder->summary);
-	} else {
-		uids = camel_folder_summary_get_changed (folder->summary);
-	}
-	if (!uids || !uids->len) {
-		camel_folder_summary_free_array (uids);
+	uids = camel_folder_summary_get_changed (folder->summary);
+	if (!uids->len) {
+		camel_folder_free_uids (folder, uids);
 		return TRUE;
 	}
 
 	for (i = 0; success && i < uids->len; i++) {
-		guint32 flags_changed, flags_set;
+		guint32 flags_changed;
 		CamelEwsMessageInfo *mi = (gpointer) camel_folder_summary_get (folder->summary, uids->pdata[i]);
 		if (!mi)
 			continue;
 
-		flags_set = camel_message_info_flags (mi);
-		flags_changed = mi->server_flags ^ flags_set;
+		flags_changed = mi->server_flags ^ mi->info.flags;
 
 		/* Exchange doesn't seem to have a sane representation
 		 * for most flags — not even replied/forwarded. */
-		if ((flags_set & CAMEL_MESSAGE_FOLDER_FLAGGED) != 0 &&
-		    (flags_changed & (CAMEL_MESSAGE_SEEN | CAMEL_MESSAGE_ANSWERED | CAMEL_MESSAGE_FORWARDED | CAMEL_MESSAGE_FLAGGED)) != 0) {
+		if (flags_changed & (CAMEL_MESSAGE_SEEN | CAMEL_MESSAGE_ANSWERED | CAMEL_MESSAGE_FORWARDED | CAMEL_MESSAGE_FLAGGED)) {
 			mi_list = g_slist_append (mi_list, mi);
 			mi_list_len++;
 
-			if (flags_set & CAMEL_MESSAGE_DELETED)
+			if (flags_changed & CAMEL_MESSAGE_DELETED)
 				deleted_uids = g_slist_prepend (deleted_uids, (gpointer) camel_pstring_strdup (uids->pdata[i]));
-			else if (flags_set & CAMEL_MESSAGE_JUNK)
-				junk_uids = g_slist_prepend (junk_uids, (gpointer) camel_pstring_strdup (uids->pdata[i]));
-		} else if (flags_set & CAMEL_MESSAGE_DELETED) {
+		} else if (flags_changed & CAMEL_MESSAGE_DELETED) {
 			deleted_uids = g_slist_prepend (deleted_uids, (gpointer) camel_pstring_strdup (uids->pdata[i]));
 			camel_message_info_free (mi);
-		} else if (flags_set & CAMEL_MESSAGE_JUNK) {
-			junk_uids = g_slist_prepend (junk_uids, (gpointer) camel_pstring_strdup (uids->pdata[i]));
-			camel_message_info_free (mi);
-		} else if ((flags_set & CAMEL_MESSAGE_FOLDER_FLAGGED) != 0) {
+		} else {
 			/* OK, the change must have been the labels */
 			mi_list = g_slist_append (mi_list, mi);
 			mi_list_len++;
-		} else {
-			camel_message_info_free (mi);
 		}
 
 		if (mi_list_len == EWS_MAX_FETCH_COUNT) {
@@ -1000,18 +870,11 @@ ews_synchronize_sync (CamelFolder *folder,
 	if (mi_list_len)
 		success = ews_sync_mi_flags (folder, mi_list, cancellable, error);
 
-	if (deleted_uids && success)
-		success = ews_delete_messages (folder, deleted_uids, ews_folder_is_of_type (folder, CAMEL_FOLDER_TYPE_TRASH), cancellable, error);
-	else
-		g_slist_free_full (deleted_uids, (GDestroyNotify) camel_pstring_free);
-
-	if (junk_uids && success)
-		success = ews_move_to_junk_folder (folder, junk_uids, cancellable, error);
-	else
-		g_slist_free_full (junk_uids, (GDestroyNotify) camel_pstring_free);
+	if (deleted_uids)
+		success = ews_delete_messages (folder, deleted_uids, ews_folder_is_trash (folder), cancellable, error);
 
 	camel_folder_summary_save_to_db (folder->summary, NULL);
-	camel_folder_summary_free_array (uids);
+	camel_folder_free_uids (folder, uids);
 
 	return success;
 }
@@ -1123,13 +986,9 @@ sync_updated_items (CamelEwsFolder *ews_folder,
                     GCancellable *cancellable,
                     GError **error)
 {
-	CamelEwsStore *ews_store;
 	CamelFolder *folder = (CamelFolder *) ews_folder;
 	GSList *items = NULL, *l;
 	GSList *generic_item_ids = NULL, *msg_ids = NULL;
-	GError *local_error = NULL;
-
-	ews_store = CAMEL_EWS_STORE (camel_folder_get_parent_store (folder));
 
 	for (l = updated_items; l != NULL; l = g_slist_next (l)) {
 		EEwsItem *item = (EEwsItem *) l->data;
@@ -1163,32 +1022,24 @@ sync_updated_items (CamelEwsFolder *ews_folder,
 
 	if (msg_ids)
 		e_ews_connection_get_items
-			(cnc, EWS_PRIORITY_MEDIUM,
+			(g_object_ref (cnc), EWS_PRIORITY_MEDIUM,
 			 msg_ids, "IdOnly", SUMMARY_MESSAGE_FLAGS,
 			 FALSE, NULL, &items, NULL, NULL,
-			 cancellable, &local_error);
+			 cancellable, error);
 
 	camel_ews_utils_sync_updated_items (ews_folder, items);
 	items = NULL;
-	if (local_error) {
-		camel_ews_store_maybe_disconnect (ews_store, local_error);
-		g_propagate_error (error, local_error);
+	if (*error)
 		goto exit;
-	}
 
 	if (generic_item_ids)
 		e_ews_connection_get_items
-			(cnc, EWS_PRIORITY_MEDIUM,
+			(g_object_ref (cnc), EWS_PRIORITY_MEDIUM,
 			 generic_item_ids, "IdOnly", SUMMARY_ITEM_FLAGS,
 			 FALSE, NULL, &items, NULL, NULL,
-			 cancellable, &local_error);
+			 cancellable, error);
 	camel_ews_utils_sync_updated_items (ews_folder, items);
 
-	if (local_error) {
-		camel_ews_store_maybe_disconnect (ews_store, local_error);
-		g_propagate_error (error, local_error);
-	}
-
 exit:
 	if (msg_ids) {
 		g_slist_foreach (msg_ids, (GFunc) g_free, NULL);
@@ -1208,12 +1059,8 @@ sync_created_items (CamelEwsFolder *ews_folder,
                     GCancellable *cancellable,
                     GError **error)
 {
-	CamelEwsStore *ews_store;
 	GSList *items = NULL, *l;
 	GSList *generic_item_ids = NULL, *msg_ids = NULL, *post_item_ids = NULL;
-	GError *local_error = NULL;
-
-	ews_store = CAMEL_EWS_STORE (camel_folder_get_parent_store (CAMEL_FOLDER (ews_folder)));
 
 	for (l = created_items; l != NULL; l = g_slist_next (l)) {
 		EEwsItem *item = (EEwsItem *) l->data;
@@ -1246,49 +1093,39 @@ sync_created_items (CamelEwsFolder *ews_folder,
 
 	if (msg_ids)
 		e_ews_connection_get_items
-			(cnc, EWS_PRIORITY_MEDIUM,
+			(g_object_ref (cnc), EWS_PRIORITY_MEDIUM,
 			 msg_ids, "IdOnly", SUMMARY_MESSAGE_PROPS,
 			 FALSE, NULL, &items, NULL, NULL,
-			 cancellable, &local_error);
+			 cancellable, error);
 
-	if (local_error) {
-		camel_ews_store_maybe_disconnect (ews_store, local_error);
-		g_propagate_error (error, local_error);
+	if (*error)
 		goto exit;
-	}
 
-	camel_ews_utils_sync_created_items (ews_folder, cnc, items);
+	camel_ews_utils_sync_created_items (ews_folder, items);
 	items = NULL;
 
 	if (post_item_ids)
 		e_ews_connection_get_items
-			(cnc, EWS_PRIORITY_MEDIUM,
+			(g_object_ref (cnc), EWS_PRIORITY_MEDIUM,
 			 post_item_ids, "IdOnly", SUMMARY_POSTITEM_PROPS,
 			 FALSE, NULL, &items, NULL, NULL,
-			 cancellable, &local_error);
+			 cancellable, error);
 
-	if (local_error) {
-		camel_ews_store_maybe_disconnect (ews_store, local_error);
-		g_propagate_error (error, local_error);
+	if (*error)
 		goto exit;
-	}
 
-	camel_ews_utils_sync_created_items (ews_folder, cnc, items);
+	camel_ews_utils_sync_created_items (ews_folder, items);
 	items = NULL;
 
 	if (generic_item_ids)
 		e_ews_connection_get_items
-			(cnc, EWS_PRIORITY_MEDIUM,
+			(g_object_ref (cnc), EWS_PRIORITY_MEDIUM,
 			 generic_item_ids, "IdOnly", SUMMARY_ITEM_PROPS,
 			 FALSE, NULL, &items, NULL, NULL,
-			 cancellable, &local_error);
+			 cancellable, error);
 
-	camel_ews_utils_sync_created_items (ews_folder, cnc, items);
+	camel_ews_utils_sync_created_items (ews_folder, items);
 
-	if (local_error) {
-		camel_ews_store_maybe_disconnect (ews_store, local_error);
-		g_propagate_error (error, local_error);
-	}
 exit:
 	if (msg_ids) {
 		g_slist_foreach (msg_ids, (GFunc) g_free, NULL);
@@ -1319,7 +1156,7 @@ ews_refresh_info_sync (CamelFolder *folder,
 	gchar *id;
 	gchar *sync_state;
 	gboolean includes_last_item = FALSE;
-	GError *local_error = NULL;
+	GError *rerror = NULL;
 
 	full_name = camel_folder_get_full_name (folder);
 	ews_store = (CamelEwsStore *) camel_folder_get_parent_store (folder);
@@ -1341,10 +1178,6 @@ ews_refresh_info_sync (CamelFolder *folder,
 	g_mutex_unlock (priv->state_lock);
 
 	cnc = camel_ews_store_get_connection (ews_store);
-	g_return_val_if_fail (cnc != NULL, FALSE);
-
-	camel_folder_summary_prepare_fetch_all (folder->summary, NULL);
-
 	id = camel_ews_store_summary_get_folder_id_from_name
 						(ews_store->summary,
 						 full_name);
@@ -1368,20 +1201,18 @@ ews_refresh_info_sync (CamelFolder *folder,
 							 "IdOnly", NULL,
 							 EWS_MAX_FETCH_COUNT, &includes_last_item,
 							 &items_created, &items_updated,
-							 &items_deleted, cancellable, &local_error);
+							 &items_deleted, cancellable, &rerror);
 
-		if (local_error) {
-			camel_ews_store_maybe_disconnect (ews_store, local_error);
+		if (rerror)
 			break;
-		}
 
 		if (items_deleted)
 			camel_ews_utils_sync_deleted_items (ews_folder, items_deleted);
 
 		if (items_created)
-			sync_created_items (ews_folder, cnc, items_created, cancellable, &local_error);
+			sync_created_items (ews_folder, cnc, items_created, cancellable, &rerror);
 
-		if (local_error) {
+		if (rerror) {
 			if (items_updated) {
 				g_slist_foreach (items_updated, (GFunc) g_object_unref, NULL);
 				g_slist_free (items_updated);
@@ -1391,9 +1222,9 @@ ews_refresh_info_sync (CamelFolder *folder,
 		}
 
 		if (items_updated)
-			sync_updated_items (ews_folder, cnc, items_updated, cancellable, &local_error);
+			sync_updated_items (ews_folder, cnc, items_updated, cancellable, &rerror);
 
-		if (local_error)
+		if (rerror)
 			break;
 
 		total = camel_folder_summary_count (folder->summary);
@@ -1407,12 +1238,12 @@ ews_refresh_info_sync (CamelFolder *folder,
 		((CamelEwsSummary *) folder->summary)->sync_state = sync_state;
 
 		camel_folder_summary_touch (folder->summary);
-	} while (!local_error && !includes_last_item);
+		camel_folder_summary_save_to_db (folder->summary, NULL);
 
-	camel_folder_summary_save_to_db (folder->summary, NULL);
+	} while (!rerror && !includes_last_item);
 
-	if (local_error)
-		g_propagate_error (error, local_error);
+	if (rerror)
+		g_propagate_error (error, rerror);
 
 	g_mutex_lock (priv->state_lock);
 	priv->refreshing = FALSE;
@@ -1422,7 +1253,7 @@ ews_refresh_info_sync (CamelFolder *folder,
 	g_object_unref (cnc);
 	g_free (id);
 
-	return !local_error;
+	return !rerror;
 }
 
 static gboolean
@@ -1439,7 +1270,6 @@ ews_append_message_sync (CamelFolder *folder,
 	CamelAddress *from;
 	CamelEwsStore *ews_store;
 	EEwsConnection *cnc;
-	GError *local_error = NULL;
 
 	ews_store = (CamelEwsStore *) camel_folder_get_parent_store (folder);
 
@@ -1451,10 +1281,6 @@ ews_append_message_sync (CamelFolder *folder,
 
 	from = CAMEL_ADDRESS (camel_mime_message_get_from (message));
 
-	if (!camel_ews_store_connected (ews_store, error)) {
-		return FALSE;
-	}
-
 	cnc = camel_ews_store_get_connection (ews_store);
 
 	if (!cnc) {
@@ -1467,9 +1293,7 @@ ews_append_message_sync (CamelFolder *folder,
 						  message,
 						  camel_message_info_flags (info),
 						  from, &itemid, &changekey,
-						  cancellable, &local_error)) {
-		camel_ews_store_maybe_disconnect (ews_store, local_error);
-		g_propagate_error (error, local_error);
+						  cancellable, error)) {
 		g_free (folder_id);
 		g_object_unref (cnc);
 		return FALSE;
@@ -1504,7 +1328,7 @@ ews_transfer_messages_to_sync (CamelFolder *source,
 	CamelFolderChangeInfo *changes = NULL;
 	const gchar *dst_full_name;
 	gchar *dst_id;
-	GError *local_error = NULL;
+	GError *rerror = NULL;
 	GSList *ids = NULL, *ret_items = NULL;
 	gint i = 0;
 
@@ -1523,44 +1347,34 @@ ews_transfer_messages_to_sync (CamelFolder *source,
 		ids = g_slist_append (ids, (gchar *) uids->pdata[i]);
 	}
 
-	camel_service_lock (CAMEL_SERVICE (dst_ews_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 	if (e_ews_connection_move_items	(cnc, EWS_PRIORITY_MEDIUM,
 					 dst_id, !delete_originals,
 					 ids, &ret_items,
-					 cancellable, &local_error)) {
-		camel_service_unlock (CAMEL_SERVICE (dst_ews_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
-
+					 cancellable, &rerror)) {
 		if (delete_originals) {
 			changes = camel_folder_change_info_new ();
 			for (i = 0; i < uids->len; i++) {
 				camel_folder_summary_remove_uid (source->summary, uids->pdata[i]);
 				camel_folder_change_info_remove_uid (changes, uids->pdata[i]);
 			}
-			if (camel_folder_change_info_changed (changes)) {
-				camel_folder_summary_touch (source->summary);
-				camel_folder_changed (source, changes);
-			}
+			camel_folder_changed (source, changes);
 			camel_folder_change_info_free (changes);
 		}
 
 		/*update the store about the content of the source and destination folders*/
 		ews_refresh_info_sync (source, cancellable, NULL);
 		ews_refresh_info_sync (destination, cancellable, NULL);
-	} else {
-		camel_service_unlock (CAMEL_SERVICE (dst_ews_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 	}
 	g_free (dst_id);
 
-	if (local_error) {
-		camel_ews_store_maybe_disconnect (dst_ews_store, local_error);
-		g_propagate_error (error, local_error);
-	}
+	if (rerror)
+		g_propagate_error (error, rerror);
 
 	g_object_unref (cnc);
 	g_slist_free (ids);
-	g_slist_free_full (ret_items, g_object_unref);
+	g_slist_free (ret_items);
 
-	return !local_error;
+	return !rerror;
 }
 
 static gboolean
@@ -1583,30 +1397,26 @@ ews_delete_messages (CamelFolder *folder,
 	ews_store = CAMEL_EWS_STORE (parent_store);
 	deleted_head = deleted_items;
 
-	if (!camel_ews_store_connected (ews_store, error)) {
-		return FALSE;
-	}
-
 	cnc = camel_ews_store_get_connection (ews_store);
 	changes = camel_folder_change_info_new ();
 
 	if (deleted_items) {
-		GError *local_error = NULL;
+		GError *rerror = NULL;
 		EwsDeleteType delete_type;
 
 		delete_type = expunge ? EWS_HARD_DELETE : EWS_MOVE_TO_DELETED_ITEMS;
 
 		camel_service_lock (CAMEL_SERVICE (ews_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 		status = e_ews_connection_delete_items (cnc, EWS_PRIORITY_MEDIUM, deleted_items, delete_type,
-							EWS_SEND_TO_NONE, FALSE, cancellable, &local_error);
+							EWS_SEND_TO_NONE, FALSE, cancellable, &rerror);
 		camel_service_unlock (CAMEL_SERVICE (ews_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 
-		if (!status && local_error->code == EWS_CONNECTION_ERROR_ITEMNOTFOUND) {
+		if (!status && rerror->code == EWS_CONNECTION_ERROR_ITEMNOTFOUND) {
 			/* If delete failed due to the item not found, ignore the error,
 			 * trigger folder info refresh and then go on to clear the
 			 * cache of the deleted items anyway. */
-			g_clear_error (&local_error);
-			status = ews_refresh_info_sync (folder, cancellable, &local_error);
+			g_clear_error (&rerror);
+			status = ews_refresh_info_sync (folder, cancellable, &rerror);
 		}
 
 		if (status) {
@@ -1621,22 +1431,16 @@ ews_delete_messages (CamelFolder *folder,
 			}
 		}
 
-		if (local_error) {
-			camel_ews_store_maybe_disconnect (ews_store, local_error);
-			g_propagate_error (error, local_error);
-		}
+		if (rerror)
+			g_propagate_error (error, rerror);
 
-		if (camel_folder_change_info_changed (changes)) {
-			camel_folder_summary_touch (folder->summary);
-			camel_folder_changed (folder, changes);
-		}
+		camel_folder_changed (folder, changes);
 
 		g_slist_foreach (deleted_head, (GFunc) camel_pstring_free, NULL);
 		g_slist_free (deleted_head);
 	}
 
 	camel_folder_change_info_free (changes);
-	g_object_unref (cnc);
 
 	return status;
 }
@@ -1661,7 +1465,7 @@ ews_expunge_sync (CamelFolder *folder,
 	if (!camel_ews_store_connected (ews_store, error))
 		return FALSE;
 
-	is_trash = ews_folder_is_of_type (folder, CAMEL_FOLDER_TYPE_TRASH);
+	is_trash = ews_folder_is_trash (folder);
 
 	camel_folder_summary_prepare_fetch_all (folder->summary, NULL);
 	known_uids = camel_folder_summary_get_array (folder->summary);
