@@ -62,11 +62,13 @@
 #define FINFO_REFRESH_INTERVAL 60
 
 struct _CamelEwsStorePrivate {
+
 	time_t last_refresh_time;
 	GMutex *get_finfo_lock;
 	EEwsConnection *cnc;
 };
 
+extern CamelServiceAuthType camel_ews_password_authtype; /*for the query_auth_types function */
 static gboolean	ews_store_construct	(CamelService *service, CamelSession *session,
 					 CamelProvider *provider, GError **error);
 
@@ -172,7 +174,6 @@ ews_store_construct (CamelService *service,
 	/* Disable virtual trash and junk folders. Exchange has real
 	 * folders for that */
 	((CamelStore *) ews_store)->flags &= ~(CAMEL_STORE_VTRASH | CAMEL_STORE_VJUNK);
-	((CamelStore *) ews_store)->flags |= CAMEL_STORE_REAL_JUNK_FOLDER;
 
 	/*storage path*/
 	session_storage_path = g_strdup (camel_service_get_user_cache_dir (service));
@@ -293,12 +294,12 @@ ews_connect_sync (CamelService *service,
 	}
 
 	priv->cnc = e_ews_connection_new (hosturl, user, NULL,
-					  g_strcmp0 ("PLAIN", camel_network_settings_get_auth_mechanism (network_settings)) != 0,
 					  G_CALLBACK (ews_store_authenticate), service,
 					  error);
 
 	if (!priv->cnc) {
 		camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
+		camel_service_disconnect_sync (service, TRUE, NULL);
 		return FALSE;
 	}
 
@@ -311,6 +312,7 @@ ews_connect_sync (CamelService *service,
 		g_object_unref (priv->cnc);
 		priv->cnc = NULL;
 		camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
+		camel_service_disconnect_sync (service, TRUE, NULL);
 		return FALSE;
 	}
 
@@ -331,19 +333,21 @@ ews_disconnect_sync (CamelService *service,
 	CamelEwsStore *ews_store = (CamelEwsStore *) service;
 	CamelServiceClass *service_class;
 
+	service_class = CAMEL_SERVICE_CLASS (camel_ews_store_parent_class);
+	if (!service_class->disconnect_sync (service, clean, cancellable, error))
+		return FALSE;
+
 	camel_service_lock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 
 	/* TODO cancel all operations in the connection */
 	if (ews_store->priv->cnc) {
-		e_ews_connection_forget_password (ews_store->priv->cnc);
 		g_object_unref (ews_store->priv->cnc);
 		ews_store->priv->cnc = NULL;
 	}
 
 	camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 
-	service_class = CAMEL_SERVICE_CLASS (camel_ews_store_parent_class);
-	return service_class->disconnect_sync (service, clean, cancellable, error);
+	return TRUE;
 }
 
 typedef struct {
@@ -357,7 +361,7 @@ static SystemFolder system_folder[] = {
 	{"deleteditems", CAMEL_FOLDER_SYSTEM | CAMEL_FOLDER_TYPE_TRASH},
 	{"drafts", CAMEL_FOLDER_SYSTEM},
 	{"inbox", CAMEL_FOLDER_SYSTEM | CAMEL_FOLDER_TYPE_INBOX},
-	{"journal", CAMEL_FOLDER_SYSTEM},
+	{"journal", CAMEL_FOLDER_SYSTEM | CAMEL_FOLDER_TYPE_MEMOS},
 	{"notes", CAMEL_FOLDER_SYSTEM | CAMEL_FOLDER_TYPE_MEMOS},
 	{"outbox", CAMEL_FOLDER_SYSTEM | CAMEL_FOLDER_TYPE_OUTBOX},
 	{"sentitems", CAMEL_FOLDER_SYSTEM | CAMEL_FOLDER_TYPE_SENT},
@@ -433,11 +437,10 @@ ews_authenticate_sync (CamelService *service,
 		/* This consumes all allocated result data. */
 		ews_update_folder_hierarchy (
 			ews_store, sync_state, includes_last_folder,
-			folders_created, folders_updated, folders_deleted);
+			folders_created, folders_deleted, folders_updated);
 	} else {
-		g_free (sync_state);
-
 		/* Make sure we're not leaking anything. */
+		g_warn_if_fail (sync_state == NULL);
 		g_warn_if_fail (folders_created == NULL);
 		g_warn_if_fail (folders_updated == NULL);
 		g_warn_if_fail (folders_deleted == NULL);
@@ -481,11 +484,9 @@ ews_authenticate_sync (CamelService *service,
 		result = CAMEL_AUTHENTICATION_ACCEPTED;
 	} else if (g_error_matches (local_error, EWS_CONNECTION_ERROR, EWS_CONNECTION_ERROR_AUTHENTICATION_FAILED)) {
 		g_clear_error (&local_error);
-		e_ews_connection_forget_password (priv->cnc);
 		result = CAMEL_AUTHENTICATION_REJECTED;
 	} else {
 		g_propagate_error (error, local_error);
-		e_ews_connection_forget_password (priv->cnc);
 		result = CAMEL_AUTHENTICATION_ERROR;
 	}
 
@@ -497,9 +498,11 @@ ews_store_query_auth_types_sync (CamelService *service,
                                  GCancellable *cancellable,
                                  GError **error)
 {
-	g_set_error_literal (error, CAMEL_ERROR, CAMEL_ERROR_GENERIC, _("Query for authentication types is not supported"));
+	GList *auth_types = NULL;
 
-	return NULL;
+	d(printf("in query auth types\n"));
+	auth_types = g_list_prepend (auth_types,  &camel_ews_password_authtype);
+	return auth_types;
 }
 
 static CamelFolderInfo * ews_create_folder_sync (CamelStore *store, const gchar *parent_name,const gchar *folder_name, GCancellable *cancellable, GError **error);
@@ -624,8 +627,6 @@ ews_folder_hierarchy_ready_cb (GObject *obj,
 	if (error != NULL) {
 		g_warning ("Unable to fetch the folder hierarchy: %s :%d \n", error->message, error->code);
 
-		camel_ews_store_maybe_disconnect (ews_store, error);
-
 		g_mutex_lock (priv->get_finfo_lock);
 		ews_store->priv->last_refresh_time -= FINFO_REFRESH_INTERVAL;
 		g_mutex_unlock (priv->get_finfo_lock);
@@ -678,7 +679,6 @@ ews_get_folder_info_sync (CamelStore *store,
 	GSList *folders_created = NULL, *folders_updated = NULL;
 	GSList *folders_deleted = NULL;
 	gboolean includes_last_folder;
-	GError *local_error = NULL;
 
 	ews_store = (CamelEwsStore *) store;
 	priv = ews_store->priv;
@@ -708,16 +708,13 @@ ews_get_folder_info_sync (CamelStore *store,
 	if (!e_ews_connection_sync_folder_hierarchy (ews_store->priv->cnc, EWS_PRIORITY_MEDIUM,
 						    &sync_state, &includes_last_folder,
 						    &folders_created, &folders_updated,
-						    &folders_deleted, cancellable, &local_error)) {
-		if (local_error)
+						    &folders_deleted, cancellable, error)) {
+		if (error)
 			g_warning ("Unable to fetch the folder hierarchy: %s :%d \n",
-				   local_error->message, local_error->code);
+				   (*error)->message, (*error)->code);
 		else
 			g_warning ("Unable to fetch the folder hierarchy.\n");
 
-		camel_ews_store_maybe_disconnect (ews_store, local_error);
-		g_propagate_error (error, local_error);
-
 		g_mutex_unlock (priv->get_finfo_lock);
 		return NULL;
 	}
@@ -743,7 +740,6 @@ ews_create_folder_sync (CamelStore *store,
 	gchar *full_name;
 	EwsFolderId *folder_id;
 	CamelFolderInfo *fi = NULL;
-	GError *local_error = NULL;
 
 	/* Get Parent folder ID */
 	if (parent_name && parent_name[0]) {
@@ -756,7 +752,9 @@ ews_create_folder_sync (CamelStore *store,
 		}
 	}
 
-	if (!camel_ews_store_connected (ews_store, error)) {
+	if (!ews_store->priv->cnc) {
+		g_set_error (error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
+			     _("Cant perform actions on the folder while in offline mode"));
 		if (fid) g_free (fid);
 		return NULL;
 	}
@@ -765,9 +763,7 @@ ews_create_folder_sync (CamelStore *store,
 	if (!e_ews_connection_create_folder (ews_store->priv->cnc,
 					     EWS_PRIORITY_MEDIUM, fid,
 					     FALSE, folder_name, &folder_id,
-					     cancellable, &local_error)) {
-		camel_ews_store_maybe_disconnect (ews_store, local_error);
-		g_propagate_error (error, local_error);
+					     cancellable, error)) {
 		g_free (fid);
 		return NULL;
 	}
@@ -803,7 +799,6 @@ ews_delete_folder_sync (CamelStore *store,
 	CamelEwsStoreSummary *ews_summary = ews_store->summary;
 	gchar *fid;
 	CamelFolderInfo *fi = NULL;
-	GError *local_error = NULL;
 
 	fid = camel_ews_store_summary_get_folder_id_from_name (ews_summary,
 							       folder_name);
@@ -813,7 +808,9 @@ ews_delete_folder_sync (CamelStore *store,
 		return FALSE;
 	}
 
-	if (!camel_ews_store_connected (ews_store, error)) {
+	if (!ews_store->priv->cnc) {
+		g_set_error (error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
+			     _("Cant perform actions on the folder while in offline mode"));
 		g_free (fid);
 		return FALSE;
 	}
@@ -821,9 +818,7 @@ ews_delete_folder_sync (CamelStore *store,
 	if (!e_ews_connection_delete_folder (ews_store->priv->cnc,
 					     EWS_PRIORITY_MEDIUM,
 					     fid, FALSE, "HardDelete",
-					     cancellable, &local_error)) {
-		camel_ews_store_maybe_disconnect (ews_store, local_error);
-		g_propagate_error (error, local_error);
+					     cancellable, error)) {
 		g_free (fid);
 		return FALSE;
 	}
@@ -879,12 +874,13 @@ ews_rename_folder_sync (CamelStore *store,
 	gchar *fid;
 	gchar *changekey;
 	gboolean res = FALSE;
-	GError *local_error = NULL;
 
 	if (!strcmp (old_name, new_name))
 		return TRUE;
 
-	if (!camel_ews_store_connected (ews_store, error)) {
+	if (!ews_store->priv->cnc) {
+		g_set_error (error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
+			     _("Cant perform actions on the folder while in offline mode"));
 		return FALSE;
 	}
 
@@ -951,7 +947,7 @@ ews_rename_folder_sync (CamelStore *store,
 		rename_data->change_key = changekey;
 
 		if (!e_ews_connection_update_folder (ews_store->priv->cnc, EWS_PRIORITY_MEDIUM,
-						     rename_folder_cb, rename_data, cancellable, &local_error)) {
+						     rename_folder_cb, rename_data, cancellable, error)) {
 			g_free (rename_data);
 			goto out;
 		}
@@ -977,7 +973,7 @@ ews_rename_folder_sync (CamelStore *store,
 			}
 		}
 		if (!e_ews_connection_move_folder (ews_store->priv->cnc, EWS_PRIORITY_MEDIUM,
-						   pfid, fid, cancellable, &local_error)) {
+						   pfid, fid, cancellable, error)) {
 			g_free (pfid);
 			goto out;
 		}
@@ -987,11 +983,6 @@ ews_rename_folder_sync (CamelStore *store,
 
 	res = TRUE;
  out:
-	if (local_error) {
-		camel_ews_store_maybe_disconnect (ews_store, local_error);
-		g_propagate_error (error, local_error);
-	}
-
 	g_free (changekey);
 	g_free (fid);
 	return res;
@@ -1026,47 +1017,30 @@ camel_ews_store_get_connection (CamelEwsStore *ews_store)
 }
 
 static CamelFolder *
-ews_get_folder_by_type_sync (CamelStore *store,
-			     guint32 folder_type,
-			     GCancellable *cancellable,
-                             GError **error)
+ews_get_trash_folder_sync (CamelStore *store,
+                           GCancellable *cancellable,
+                           GError **error)
 {
 	CamelEwsStore *ews_store;
-	CamelFolder *folder = NULL;
+	CamelFolder *trash = NULL;
 	gchar *folder_id, *folder_name;
 
 	g_return_val_if_fail (CAMEL_IS_EWS_STORE (store), NULL);
 
 	ews_store = CAMEL_EWS_STORE (store);
-	folder_id = camel_ews_store_summary_get_folder_id_from_folder_type (ews_store->summary, folder_type);
+	folder_id = camel_ews_store_summary_get_folder_id_from_folder_type (ews_store->summary, CAMEL_FOLDER_TYPE_TRASH);
 
 	if (!folder_id)
 		return NULL;
 
 	folder_name = camel_ews_store_summary_get_folder_full_name (ews_store->summary, folder_id, NULL);
 
-	folder = ews_get_folder_sync (store, folder_name, 0, cancellable, error);
+	trash = ews_get_folder_sync (store, folder_name, 0, cancellable, error);
 
 	g_free (folder_name);
 	g_free (folder_id);
 
-	return folder;
-}
-
-static CamelFolder *
-ews_get_trash_folder_sync (CamelStore *store,
-                           GCancellable *cancellable,
-                           GError **error)
-{
-	return ews_get_folder_by_type_sync (store, CAMEL_FOLDER_TYPE_TRASH, cancellable, error);
-}
-
-static CamelFolder *
-ews_get_junk_folder_sync (CamelStore *store,
-                          GCancellable *cancellable,
-                          GError **error)
-{
-	return ews_get_folder_by_type_sync (store, CAMEL_FOLDER_TYPE_JUNK, cancellable, error);
+	return trash;
 }
 
 static gboolean
@@ -1101,27 +1075,6 @@ camel_ews_store_connected (CamelEwsStore *ews_store,
 	return TRUE;
 }
 
-void
-camel_ews_store_maybe_disconnect (CamelEwsStore *store,
-				  const GError *error)
-{
-	CamelService *service;
-
-	g_return_if_fail (store != NULL);
-
-	if (!error)
-		return;
-
-	service = CAMEL_SERVICE (store);
-
-	if (camel_service_get_connection_status (service) != CAMEL_SERVICE_CONNECTED)
-		return;
-
-	if (g_error_matches (error, EWS_CONNECTION_ERROR, EWS_CONNECTION_ERROR_NORESPONSE) ||
-	    g_error_matches (error, EWS_CONNECTION_ERROR, EWS_CONNECTION_ERROR_AUTHENTICATION_FAILED))
-		camel_service_disconnect_sync (service, FALSE, NULL);
-}
-
 static void
 ews_store_dispose (GObject *object)
 {
@@ -1190,7 +1143,6 @@ camel_ews_store_class_init (CamelEwsStoreClass *class)
 	store_class->free_folder_info = camel_store_free_folder_info_full;
 
 	store_class->get_trash_folder_sync = ews_get_trash_folder_sync;
-	store_class->get_junk_folder_sync = ews_get_junk_folder_sync;
 	store_class->can_refresh_folder = ews_can_refresh_folder;
 }
 
