@@ -521,7 +521,6 @@ struct add_esrc_data {
 	gchar *username;
 	gchar *email_id;
 	gchar *hosturl;
-	gchar *ews_auth_type;
 	gint refresh_timeout;
 };
 
@@ -532,8 +531,7 @@ static gboolean ews_do_add_esource (gpointer user_data)
 	ews_esource_utils_add_esource (add_data->folder, add_data->account_uri,
 				       add_data->account_name,
 				       add_data->username, add_data->email_id,
-				       add_data->hosturl, add_data->refresh_timeout,
-				       add_data->ews_auth_type);
+				       add_data->hosturl, add_data->refresh_timeout);
 
 	g_object_unref (add_data->folder);
 	g_free (add_data->account_uri);
@@ -541,7 +539,6 @@ static gboolean ews_do_add_esource (gpointer user_data)
 	g_free (add_data->username);
 	g_free (add_data->email_id);
 	g_free (add_data->hosturl);
-	g_free (add_data->ews_auth_type);
 	g_free (add_data);
 
 	return FALSE;
@@ -590,7 +587,6 @@ sync_created_folders (CamelEwsStore *ews_store,
 			/* Duplicate... for now */
 			add_data->email_id = g_strdup (email);
 			add_data->hosturl = g_strdup (hosturl);
-			add_data->ews_auth_type = g_strdup (camel_network_settings_get_auth_mechanism (network_settings));
 			/* FIXME pass right refresh timeout */
 
 			camel_url_free (url);
@@ -668,11 +664,7 @@ camel_ews_utils_sync_deleted_items (CamelEwsFolder *ews_folder,
 	camel_db_delete_uids (((CamelStore *) ews_store)->cdb_w, full_name, items_deleted_list, NULL);
 	g_list_free (items_deleted_list);
 
-	if (camel_folder_change_info_changed (ci)) {
-		camel_folder_summary_touch (folder->summary);
-		camel_folder_summary_save_to_db (folder->summary, NULL);
-		camel_folder_changed (folder, ci);
-	}
+	camel_folder_changed ((CamelFolder *) ews_folder, ci);
 	camel_folder_change_info_free (ci);
 
 	g_slist_foreach (items_deleted, (GFunc) g_free, NULL);
@@ -797,48 +789,38 @@ ews_utils_get_server_flags (EEwsItem *item)
 }
 
 static const gchar *
-form_email_string_from_mb (EEwsConnection *cnc,
-			   const EwsMailbox *mb,
-			   GCancellable *cancellable)
+form_email_string_from_mb (const EwsMailbox *mb)
 {
+	const gchar *ret = NULL;
+
 	if (mb) {
 		GString *str;
-		gchar *email = NULL;
-
-		if (g_strcmp0 (mb->mb_type, "EX") == 0) {
-			e_ews_connection_ex_to_smtp_sync (
-				cnc, EWS_PRIORITY_MEDIUM,
-				mb->name, mb->email, &email,
-				cancellable, NULL);
-
-			/* do not scare users with EX addresses */
-			if (!email)
-				email = g_strdup ("");
-		}
 
 		str = g_string_new ("");
 		if (mb->name && mb->name[0]) {
-			g_string_append (str, mb->name);
-			g_string_append (str, " ");
+			str = g_string_append (str, mb->name);
+			str = g_string_append (str, " ");
+		} else {
+			str = g_string_append (str, mb->email);
+			str = g_string_append (str, " ");
 		}
 
-		if (mb->email || email) {
+		if (mb->email) {
 			g_string_append (str, "<");
-			g_string_append (str, email ? email : mb->email);
+			str = g_string_append (str, mb->email);
 			g_string_append (str, ">");
 		}
 
-		g_free (email);
+		ret = camel_pstring_add (str->str, TRUE);
+		g_string_free (str, FALSE);
 
-		return camel_pstring_add (g_string_free (str, FALSE), TRUE);
+		return ret;
 	} else
 	       return camel_pstring_strdup ("");
 }
 
 static const gchar *
-form_recipient_list (EEwsConnection *cnc,
-		     const GSList *recipients,
-		     GCancellable *cancellable)
+form_recipient_list (const GSList *recipients)
 {
 	const GSList *l;
 	GString *str = NULL;
@@ -849,7 +831,7 @@ form_recipient_list (EEwsConnection *cnc,
 
 	for (l = recipients; l != NULL; l = g_slist_next (l)) {
 		EwsMailbox *mb = (EwsMailbox *) l->data;
-		const gchar *mb_str = form_email_string_from_mb (cnc, mb, cancellable);
+		const gchar *mb_str = form_email_string_from_mb (mb);
 
 		if (!str)
 			str = g_string_new ("");
@@ -980,18 +962,14 @@ camel_ews_utils_sync_updated_items (CamelEwsFolder *ews_folder,
 		g_object_unref (item);
 	}
 
-	if (camel_folder_change_info_changed (ci)) {
-		camel_folder_summary_touch (folder->summary);
-		camel_folder_summary_save_to_db (folder->summary, NULL);
-		camel_folder_changed ((CamelFolder *) ews_folder, ci);
-	}
+	camel_folder_summary_save_to_db (folder->summary, NULL);
+	camel_folder_changed ((CamelFolder *) ews_folder, ci);
 	camel_folder_change_info_free (ci);
 	g_slist_free (items_updated);
 }
 
 void
 camel_ews_utils_sync_created_items (CamelEwsFolder *ews_folder,
-				    EEwsConnection *cnc,
                                     GSList *items_created)
 {
 	CamelFolder *folder;
@@ -1055,15 +1033,13 @@ camel_ews_utils_sync_created_items (CamelEwsFolder *ews_folder,
 		mi->info.date_received = e_ews_item_get_date_received (item);
 
 		from = e_ews_item_get_from (item);
-		if (!from)
-			from = e_ews_item_get_sender (item);
-		mi->info.from = form_email_string_from_mb (cnc, from, NULL);
+		mi->info.from = form_email_string_from_mb (from);
 
 		to = e_ews_item_get_to_recipients (item);
-		mi->info.to = form_recipient_list (cnc, to, NULL);
+		mi->info.to = form_recipient_list (to);
 
 		cc = e_ews_item_get_cc_recipients (item);
-		mi->info.cc = form_recipient_list (cnc, cc, NULL);
+		mi->info.cc = form_recipient_list (cc);
 
 		e_ews_item_has_attachments (item, &has_attachments);
 		if (has_attachments)
@@ -1073,29 +1049,16 @@ camel_ews_utils_sync_created_items (CamelEwsFolder *ews_folder,
 		server_flags = ews_utils_get_server_flags (item);
 		ews_utils_merge_server_user_flags (item, mi);
 
-		mi->info.flags |= server_flags;
-		mi->server_flags = server_flags;
-
-		camel_folder_summary_add (folder->summary, (CamelMessageInfo *) mi);
-
-		/* camel_folder_summary_add() sets folder_flagged flag
-		   on the message info, but this is a fresh item downloaded
-		   from the server, thus unset it, to avoid resync up to the server
-		   on folder leave/store
-		*/
-		mi->info.flags &= ~CAMEL_MESSAGE_FOLDER_FLAGGED;
-
+		camel_ews_summary_add_message_info (folder->summary, server_flags,
+						    (CamelMessageInfo *) mi);
 		camel_folder_change_info_add_uid (ci, id->id);
 		camel_folder_change_info_recent_uid (ci, id->id);
 
 		g_object_unref (item);
 	}
 
-	if (camel_folder_change_info_changed (ci)) {
-		camel_folder_summary_touch (folder->summary);
-		camel_folder_summary_save_to_db (folder->summary, NULL);
-		camel_folder_changed ((CamelFolder *) ews_folder, ci);
-	}
+	camel_folder_summary_save_to_db (folder->summary, NULL);
+	camel_folder_changed ((CamelFolder *) ews_folder, ci);
 	camel_folder_change_info_free (ci);
 	g_slist_free (items_created);
 }
